@@ -2,10 +2,6 @@
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
-#ifdef MOZ_LOGGING
-#define FORCE_PR_LOG
-#endif
-
 
 #include "msgCore.h"    // precompiled header...
 #include "nsPop3Sink.h"
@@ -37,43 +33,36 @@
 #include "nsEmbedCID.h"
 #include "nsMsgUtils.h"
 #include "nsMsgBaseCID.h"
-#include "nsLocalStrings.h"
 #include "nsServiceManagerUtils.h"
 #include "nsIPop3Service.h"
 #include "nsMsgLocalCID.h"
 #include "mozilla/Services.h"
 
 extern PRLogModuleInfo *POP3LOGMODULE;
+#define POP3LOG(str) "%s sink: [this=%p] " str, POP3LOGMODULE->name, this
 
-NS_IMPL_THREADSAFE_ISUPPORTS1(nsPop3Sink, nsIPop3Sink)
+NS_IMPL_ISUPPORTS(nsPop3Sink, nsIPop3Sink)
 
 nsPop3Sink::nsPop3Sink()
 {
     m_authed = false;
     m_downloadingToTempFile = false;
-    m_accountUrl = nsnull;
     m_biffState = 0;
     m_numNewMessages = 0;
     m_numNewMessagesInFolder = 0;
     m_numMsgsDownloaded = 0;
     m_senderAuthed = false;
-    m_outputBuffer = nsnull;
-    m_outputBufferSize = 0;
-    m_popServer = nsnull;
-    m_outFileStream = nsnull;
+    m_outFileStream = nullptr;
     m_uidlDownload = false;
     m_buildMessageUri = false;
     if (!POP3LOGMODULE)
       POP3LOGMODULE = PR_NewLogModule("POP3");
-
 }
 
 nsPop3Sink::~nsPop3Sink()
 {
-    PR_Free(m_accountUrl);
-    PR_Free(m_outputBuffer);
-    NS_IF_RELEASE(m_popServer);
-    PR_LOG(POP3LOGMODULE, PR_LOG_MAX, ("Calling ReleaseFolderLock from ~nsPop3Sink"));
+    MOZ_LOG(POP3LOGMODULE, mozilla::LogLevel::Debug,
+            (POP3LOG("Calling ReleaseFolderLock from ~nsPop3Sink")));
     ReleaseFolderLock();
 }
 
@@ -99,30 +88,21 @@ nsPop3Sink::SetSenderAuthedFlag(void* closure, bool authed)
 }
 
 nsresult
-nsPop3Sink::SetMailAccountURL(const char* urlString)
+nsPop3Sink::SetMailAccountURL(const nsACString &urlString)
 {
-  if (urlString)
-  {
-    PR_Free(m_accountUrl);
-    m_accountUrl = PL_strdup(urlString);
-  }
-
+  m_accountUrl.Assign(urlString);
   return NS_OK;
 }
 
 nsresult
-nsPop3Sink::GetMailAccountURL(char* *urlString)
+nsPop3Sink::GetMailAccountURL(nsACString &urlString)
 {
-  NS_ASSERTION(urlString, "null getter in getMailAccountURL");
-  if (!urlString)
-    return NS_ERROR_NULL_POINTER;
-
-  *urlString = strdup(m_accountUrl);
+  urlString.Assign(m_accountUrl);
   return NS_OK;
 }
 
 partialRecord::partialRecord() :
-  m_msgDBHdr(nsnull)
+  m_msgDBHdr(nullptr)
 {
 }
 
@@ -154,7 +134,7 @@ nsPop3Sink::FindPartialMessages()
   while(hasMore && NS_SUCCEEDED(rv))
   {
     nsCOMPtr<nsISupports> aSupport;
-    PRUint32 flags = 0;
+    uint32_t flags = 0;
     rv = messages->GetNext(getter_AddRefs(aSupport));
     nsCOMPtr<nsIMsgDBHdr> msgDBHdr(do_QueryInterface(aSupport, &rv));
     msgDBHdr->GetFlags(&flags);
@@ -189,7 +169,7 @@ nsPop3Sink::FindPartialMessages()
     }
     messages->HasMoreElements(&hasMore);
   }
-  if (isOpen)
+  if (isOpen && folderScanState.m_inputStream)
     folderScanState.m_inputStream->Close();
   return rv;
 }
@@ -201,10 +181,10 @@ nsPop3Sink::FindPartialMessages()
 void
 nsPop3Sink::CheckPartialMessages(nsIPop3Protocol *protocol)
 {
-  PRUint32 count = m_partialMsgsArray.Length();
+  uint32_t count = m_partialMsgsArray.Length();
   bool deleted = false;
 
-  for (PRUint32 i = 0; i < count; i++)
+  for (uint32_t i = 0; i < count; i++)
   {
     partialRecord *partialMsg;
     bool found = true;
@@ -213,7 +193,7 @@ nsPop3Sink::CheckPartialMessages(nsIPop3Protocol *protocol)
     if (!found && partialMsg->m_msgDBHdr)
     {
       if (m_newMailParser)
-        m_newMailParser->m_mailDB->DeleteHeader(partialMsg->m_msgDBHdr, nsnull, false, true);
+        m_newMailParser->m_mailDB->DeleteHeader(partialMsg->m_msgDBHdr, nullptr, false, true);
       deleted = true;
     }
     delete partialMsg;
@@ -250,12 +230,14 @@ nsPop3Sink::BeginMailDelivery(bool uidlDownload, nsIMsgWindow *aMsgWindow, bool*
   m_folder->GetLocked(&isLocked);
   if(!isLocked)
   {
-    PR_LOG(POP3LOGMODULE, PR_LOG_MAX, ("BeginMailDelivery acquiring semaphore"));
+    MOZ_LOG(POP3LOGMODULE, mozilla::LogLevel::Debug,
+            (POP3LOG("BeginMailDelivery acquiring semaphore")));
     m_folder->AcquireSemaphore(supports);
   }
   else
   {
-    PR_LOG(POP3LOGMODULE, PR_LOG_MAX, ("BeginMailDelivery folder locked"));
+    MOZ_LOG(POP3LOGMODULE, mozilla::LogLevel::Debug,
+            (POP3LOG("BeginMailDelivery folder locked")));
     return NS_MSG_FOLDER_BUSY;
   }
   m_uidlDownload = uidlDownload;
@@ -284,7 +266,7 @@ nsPop3Sink::EndMailDelivery(nsIPop3Protocol *protocol)
   {
     if (m_outFileStream)
       m_outFileStream->Flush();  // try this.
-    m_newMailParser->OnStopRequest(nsnull, nsnull, NS_OK);
+    m_newMailParser->OnStopRequest(nullptr, nullptr, NS_OK);
     m_newMailParser->EndMsgDownload();
   }
   if (m_outFileStream)
@@ -292,11 +274,6 @@ nsPop3Sink::EndMailDelivery(nsIPop3Protocol *protocol)
     m_outFileStream->Close();
     m_outFileStream = 0;
   }
-  if (m_inboxOutputStream)
-  {
-    m_inboxOutputStream->Close();
-    m_inboxOutputStream = nsnull;
-  }
 
   if (m_downloadingToTempFile)
     m_tmpDownloadFile->Remove(false);
@@ -305,13 +282,14 @@ nsPop3Sink::EndMailDelivery(nsIPop3Protocol *protocol)
   if (m_newMailParser)
     m_newMailParser->UpdateDBFolderInfo();
 
-  PR_LOG(POP3LOGMODULE, PR_LOG_MAX, ("Calling ReleaseFolderLock from EndMailDelivery"));
+  MOZ_LOG(POP3LOGMODULE, mozilla::LogLevel::Debug,
+          (POP3LOG("Calling ReleaseFolderLock from EndMailDelivery")));
   nsresult rv = ReleaseFolderLock();
   NS_ASSERTION(NS_SUCCEEDED(rv),"folder lock not released successfully");
 
   bool filtersRun;
-  m_folder->CallFilterPlugins(nsnull, &filtersRun); // ??? do we need msgWindow?
-  PRInt32 numNewMessagesInFolder;
+  m_folder->CallFilterPlugins(nullptr, &filtersRun); // ??? do we need msgWindow?
+  int32_t numNewMessagesInFolder;
   // if filters have marked msgs read or deleted, the num new messages count
   // will go negative by the number of messages marked read or deleted,
   // so if we add that number to the number of msgs downloaded, that will give
@@ -338,7 +316,7 @@ nsPop3Sink::EndMailDelivery(nsIPop3Protocol *protocol)
   if (server)
   {
     nsCOMPtr <nsIMsgFilterList> filterList;
-    rv = server->GetFilterList(nsnull, getter_AddRefs(filterList));
+    rv = server->GetFilterList(nullptr, getter_AddRefs(filterList));
     NS_ENSURE_SUCCESS(rv, rv);
 
     if (filterList)
@@ -380,7 +358,7 @@ nsPop3Sink::EndMailDelivery(nsIPop3Protocol *protocol)
             // this will actually fix the problem.
             openFolder->GetLocked(&isLocked);
             if(!isLocked)
-              openFolder->CallFilterPlugins(nsnull, &filtersRun);
+              openFolder->CallFilterPlugins(nullptr, &filtersRun);
           }
         }
       }
@@ -404,7 +382,8 @@ nsPop3Sink::ReleaseFolderLock()
   bool haveSemaphore;
   nsCOMPtr <nsISupports> supports = do_QueryInterface(static_cast<nsIPop3Sink*>(this));
   result = m_folder->TestSemaphore(supports, &haveSemaphore);
-  PR_LOG(POP3LOGMODULE, PR_LOG_MAX, ("ReleaseFolderLock haveSemaphore = %s", haveSemaphore ? "TRUE" : "FALSE"));
+  MOZ_LOG(POP3LOGMODULE, mozilla::LogLevel::Debug,
+          (POP3LOG("ReleaseFolderLock haveSemaphore = %s"), haveSemaphore ? "TRUE" : "FALSE"));
 
   if(NS_SUCCEEDED(result) && haveSemaphore)
     result = m_folder->ReleaseSemaphore(supports);
@@ -423,11 +402,6 @@ nsPop3Sink::AbortMailDelivery(nsIPop3Protocol *protocol)
     m_outFileStream->Close();
     m_outFileStream = 0;
   }
-  if (m_inboxOutputStream)
-  {
-    m_inboxOutputStream->Close();
-    m_inboxOutputStream = nsnull;
-  }
 
   if (m_downloadingToTempFile && m_tmpDownloadFile)
     m_tmpDownloadFile->Remove(false);
@@ -436,7 +410,8 @@ nsPop3Sink::AbortMailDelivery(nsIPop3Protocol *protocol)
   we have truncated the inbox, so berkeley mailbox and msf file are in sync*/
   if (m_newMailParser)
     m_newMailParser->UpdateDBFolderInfo();
-  PR_LOG(POP3LOGMODULE, PR_LOG_MAX, ("Calling ReleaseFolderLock from AbortMailDelivery"));
+  MOZ_LOG(POP3LOGMODULE, mozilla::LogLevel::Debug,
+          (POP3LOG("Calling ReleaseFolderLock from AbortMailDelivery")));
 
   nsresult rv = ReleaseFolderLock();
   NS_ASSERTION(NS_SUCCEEDED(rv),"folder lock not released successfully");
@@ -453,7 +428,7 @@ nsPop3Sink::AbortMailDelivery(nsIPop3Protocol *protocol)
 NS_IMETHODIMP
 nsPop3Sink::IncorporateBegin(const char* uidlString,
                              nsIURI* aURL,
-                             PRUint32 flags,
+                             uint32_t flags,
                              void** closure)
 {
 #ifdef DEBUG
@@ -548,66 +523,55 @@ nsPop3Sink::IncorporateBegin(const char* uidlString,
 
   if (NS_FAILED(rv))
   {
-    m_newMailParser = nsnull;
+    m_newMailParser = nullptr;
     rv = NS_OK;
   }
-  else
-  {
-    if (m_downloadingToTempFile)
-    {
-      // Tell the parser to use the offset that will be in the dest folder,
-      // not the temp folder, so that the msg hdr will start off with
-      // the correct mdb oid
-      PRInt64 fileSize;
-      path->GetFileSize(&fileSize);
-      m_newMailParser->SetEnvelopePos((PRUint32) fileSize);
-    }
-  }
+
     if (closure)
         *closure = (void*) this;
 
-    char *dummyEnvelope = GetDummyEnvelope();
-
-  rv = WriteLineToMailbox(dummyEnvelope);
-  NS_ENSURE_SUCCESS(rv, rv);
-    // write out account-key before UIDL so the code that looks for
+    nsCString outputString(GetDummyEnvelope());
+    rv = WriteLineToMailbox(outputString);
+    NS_ENSURE_SUCCESS(rv, rv);
+    // Write out account-key before UIDL so the code that looks for
     // UIDL will find the account first and know it can stop looking
     // once it finds the UIDL line.
     if (!m_accountKey.IsEmpty())
     {
-      nsCAutoString outputString;
       outputString.AssignLiteral(HEADER_X_MOZILLA_ACCOUNT_KEY ": ");
       outputString.Append(m_accountKey);
       outputString.AppendLiteral(MSG_LINEBREAK);
-      WriteLineToMailbox(outputString.get());
+      rv = WriteLineToMailbox(outputString);
+      NS_ENSURE_SUCCESS(rv, rv);
     }
     if (uidlString)
     {
-      nsCAutoString uidlCString("X-UIDL: ");
-      uidlCString += uidlString;
-      uidlCString += MSG_LINEBREAK;
-      rv = WriteLineToMailbox(const_cast<char*>(uidlCString.get()));
+      outputString.AssignLiteral("X-UIDL: ");
+      outputString.Append(uidlString);
+      outputString.AppendLiteral(MSG_LINEBREAK);
+      rv = WriteLineToMailbox(outputString);
       NS_ENSURE_SUCCESS(rv, rv);
     }
+
     // WriteLineToMailbox("X-Mozilla-Status: 8000" MSG_LINEBREAK);
     char *statusLine = PR_smprintf(X_MOZILLA_STATUS_FORMAT MSG_LINEBREAK, flags);
-    rv = WriteLineToMailbox(statusLine);
-    if (NS_FAILED(rv)) return rv;
-    rv = WriteLineToMailbox("X-Mozilla-Status2: 00000000" MSG_LINEBREAK);
-    if (NS_FAILED(rv)) return rv;
-    // leave space for 60 bytes worth of keys/tags
-    rv = WriteLineToMailbox(X_MOZILLA_KEYWORDS);
+    outputString.Assign(statusLine);
+    rv = WriteLineToMailbox(outputString);
     PR_smprintf_free(statusLine);
+    NS_ENSURE_SUCCESS(rv, rv);
+
+    rv = WriteLineToMailbox(NS_LITERAL_CSTRING("X-Mozilla-Status2: 00000000" MSG_LINEBREAK));
+    NS_ENSURE_SUCCESS(rv, rv);
+
+    // leave space for 60 bytes worth of keys/tags
+    rv = WriteLineToMailbox(NS_LITERAL_CSTRING(X_MOZILLA_KEYWORDS));
     return NS_OK;
 }
 
 NS_IMETHODIMP
 nsPop3Sink::SetPopServer(nsIPop3IncomingServer *server)
 {
-  NS_IF_RELEASE(m_popServer);
-  m_popServer=server;
-  NS_ADDREF(m_popServer);
-
+  m_popServer = server;
   return NS_OK;
 }
 
@@ -635,8 +599,8 @@ NS_IMETHODIMP nsPop3Sink::SetFolder(nsIMsgFolder * aFolder)
 nsresult
 nsPop3Sink::GetServerFolder(nsIMsgFolder **aFolder)
 {
-  if (!aFolder)
-    return NS_ERROR_NULL_POINTER;
+  NS_ENSURE_ARG_POINTER(aFolder);
+
   if (m_popServer)
   {
     // not sure what this is used for - might be wrong if we have a deferred account.
@@ -644,11 +608,11 @@ nsPop3Sink::GetServerFolder(nsIMsgFolder **aFolder)
     if (incomingServer)
       return incomingServer->GetRootFolder(aFolder);
   }
-  *aFolder = nsnull;
+  *aFolder = nullptr;
   return NS_ERROR_NULL_POINTER;
 }
 
-NS_IMETHODIMP nsPop3Sink::SetMsgsToDownload(PRUint32 aNumMessages)
+NS_IMETHODIMP nsPop3Sink::SetMsgsToDownload(uint32_t aNumMessages)
 {
   m_numNewMessages = aNumMessages;
   return NS_OK;
@@ -677,55 +641,35 @@ nsPop3Sink::GetDummyEnvelope(void)
 
 nsresult
 nsPop3Sink::IncorporateWrite(const char* block,
-                             PRInt32 length)
+                             int32_t length)
 {
-  PRInt32 blockOffset = 0;
+  m_outputBuffer.Truncate();
   if (!strncmp(block, "From ", 5))
-  {
-    length++;
-    blockOffset = 1;
-  }
-  if (!m_outputBuffer || length > m_outputBufferSize)
-  {
-    if (!m_outputBuffer)
-      m_outputBuffer = (char*) PR_MALLOC(length+1);
-    else
-      m_outputBuffer = (char*) PR_REALLOC(m_outputBuffer, length+1);
+    m_outputBuffer.Assign('>');
 
-    m_outputBufferSize = length;
-  }
-  if (m_outputBuffer)
-  {
-    if (blockOffset == 1)
-      *m_outputBuffer = '>';
-    memcpy(m_outputBuffer + blockOffset, block, length - blockOffset);
-    *(m_outputBuffer + length) = 0;
-    nsresult rv = WriteLineToMailbox (m_outputBuffer);
-    if (NS_FAILED(rv)) return rv;
-  }
-  return NS_OK;
+  m_outputBuffer.Append(block);
+
+  return WriteLineToMailbox(m_outputBuffer);
 }
 
-nsresult nsPop3Sink::WriteLineToMailbox(const char *buffer)
+nsresult nsPop3Sink::WriteLineToMailbox(const nsACString& buffer)
 {
-
-  if (buffer)
+  if (!buffer.IsEmpty())
   {
-    PRInt32 bufferLen = PL_strlen(buffer);
-    if (m_newMailParser) // HandleLine should really take a const char *...
-      m_newMailParser->HandleLine((char *) buffer, bufferLen);
+    uint32_t bufferLen = buffer.Length();
+    if (m_newMailParser)
+      m_newMailParser->HandleLine(buffer.BeginReading(), bufferLen);
     // The following (!m_outFileStream etc) was added to make sure that we don't write somewhere
     // where for some reason or another we can't write to and lose the messages
     // See bug 62480
-    if (!m_outFileStream)
-      return NS_ERROR_OUT_OF_MEMORY;
+    NS_ENSURE_TRUE(m_outFileStream, NS_ERROR_OUT_OF_MEMORY);
 
     // seek to the end in case someone else has seeked elsewhere in our stream.
     nsCOMPtr <nsISeekableStream> seekableOutStream = do_QueryInterface(m_outFileStream);
     seekableOutStream->Seek(nsISeekableStream::NS_SEEK_END, 0);
-    PRUint32 bytesWritten;
-    m_outFileStream->Write(buffer, bufferLen, &bytesWritten);
-    if ((PRInt32) bytesWritten != bufferLen) return NS_ERROR_FAILURE;
+    uint32_t bytesWritten;
+    m_outFileStream->Write(buffer.BeginReading(), bufferLen, &bytesWritten);
+    NS_ENSURE_TRUE(bytesWritten == bufferLen, NS_ERROR_FAILURE);
   }
   return NS_OK;
 }
@@ -743,8 +687,10 @@ nsresult nsPop3Sink::HandleTempDownloadFailed(nsIMsgWindow *msgWindow)
 
   m_newMailParser->m_newMsgHdr->GetMime2DecodedSubject(subjectStr);
   m_newMailParser->m_newMsgHdr->GetMime2DecodedAuthor(fromStr);
-  const PRUnichar *params[] = { fromStr.get(), subjectStr.get() };
-  bundle->FormatStringFromID(POP3_TMP_DOWNLOAD_FAILED, params, 2, getter_Copies(confirmString));
+  const char16_t *params[] = { fromStr.get(), subjectStr.get() };
+  bundle->FormatStringFromName(
+    MOZ_UTF16("pop3TmpDownloadError"),
+    params, 2, getter_Copies(confirmString));
   nsCOMPtr<nsIDOMWindow> parentWindow;
   nsCOMPtr<nsIPromptService> promptService = do_GetService(NS_PROMPTSERVICE_CONTRACTID);
   nsCOMPtr<nsIDocShell> docShell;
@@ -755,17 +701,17 @@ nsresult nsPop3Sink::HandleTempDownloadFailed(nsIMsgWindow *msgWindow)
   }
   if (promptService && !confirmString.IsEmpty())
   {
-    PRInt32 dlgResult  = -1;
+    int32_t dlgResult  = -1;
     bool dummyValue = false;
-    rv = promptService->ConfirmEx(parentWindow, nsnull, confirmString.get(),
+    rv = promptService->ConfirmEx(parentWindow, nullptr, confirmString.get(),
                       nsIPromptService::STD_YES_NO_BUTTONS,
-                      nsnull,
-                      nsnull,
-                      nsnull,
-                      nsnull,
+                      nullptr,
+                      nullptr,
+                      nullptr,
+                      nullptr,
                       &dummyValue,
                       &dlgResult);
-    m_newMailParser->m_newMsgHdr = nsnull;
+    m_newMailParser->m_newMsgHdr = nullptr;
 
     return (dlgResult == 0) ? NS_OK : NS_MSG_ERROR_COPYING_FROM_TMP_DOWNLOAD;
   }
@@ -774,18 +720,18 @@ nsresult nsPop3Sink::HandleTempDownloadFailed(nsIMsgWindow *msgWindow)
 
 
 NS_IMETHODIMP
-nsPop3Sink::IncorporateComplete(nsIMsgWindow *aMsgWindow, PRInt32 aSize)
+nsPop3Sink::IncorporateComplete(nsIMsgWindow *aMsgWindow, int32_t aSize)
 {
   if (m_buildMessageUri && !m_baseMessageUri.IsEmpty() && m_newMailParser &&
       m_newMailParser->m_newMsgHdr)
   {
-    PRUint32 msgKey;
+    nsMsgKey msgKey;
     m_newMailParser->m_newMsgHdr->GetMessageKey(&msgKey);
     m_messageUri.Truncate();
     nsBuildLocalMessageURI(m_baseMessageUri.get(), msgKey, m_messageUri);
   }
 
-  nsresult rv = WriteLineToMailbox(MSG_LINEBREAK);
+  nsresult rv = WriteLineToMailbox(NS_LITERAL_CSTRING(MSG_LINEBREAK));
   NS_ENSURE_SUCCESS(rv, rv);
   bool leaveOnServer = false;
   m_popServer->GetLeaveMessagesOnServer(&leaveOnServer);
@@ -839,8 +785,8 @@ nsPop3Sink::IncorporateComplete(nsIMsgWindow *aMsgWindow, PRInt32 aSize)
       NS_ENSURE_SUCCESS(rv, HandleTempDownloadFailed(aMsgWindow));
       if (m_outFileStream)
       {
-        PRInt64 tmpDownloadFileSize;
-        PRUint32 msgSize;
+        int64_t tmpDownloadFileSize;
+        uint32_t msgSize;
         hdr->GetMessageSize(&msgSize);
         // we need to clone because nsLocalFileUnix caches its stat result,
         // so it doesn't realize the file has changed size.
@@ -896,13 +842,16 @@ nsPop3Sink::IncorporateAbort(bool uidlDownload)
 {
   nsresult rv = m_outFileStream->Close();
   NS_ENSURE_SUCCESS(rv,rv);
-  if (!m_downloadingToTempFile && m_msgStore)
+  if (!m_downloadingToTempFile && m_msgStore && m_newMailParser &&
+      m_newMailParser->m_newMsgHdr)
+  {
       m_msgStore->DiscardNewMessage(m_outFileStream,
                                     m_newMailParser->m_newMsgHdr);
+  }
 #ifdef DEBUG
-    printf("Incorporate message abort.\n");
+  printf("Incorporate message abort.\n");
 #endif
-    return rv;
+  return rv;
 }
 
 nsresult
@@ -915,7 +864,7 @@ nsPop3Sink::BiffGetNewMail()
 }
 
 nsresult
-nsPop3Sink::SetBiffStateAndUpdateFE(PRUint32 aBiffState, PRInt32 numNewMessages, bool notify)
+nsPop3Sink::SetBiffStateAndUpdateFE(uint32_t aBiffState, int32_t numNewMessages, bool notify)
 {
   m_biffState = aBiffState;
   if (m_newMailParser)
