@@ -9,6 +9,7 @@ const NS_ERROR_MODULE_NETWORK = 2152398848;
 const NS_ERROR_UNKNOWN_HOST = NS_ERROR_MODULE_NETWORK + 30;
 const NS_ERROR_CONNECTION_REFUSED = NS_ERROR_MODULE_NETWORK + 13;
 const NS_ERROR_NET_TIMEOUT = NS_ERROR_MODULE_NETWORK + 14;
+const NS_ERROR_OFFLINE = NS_ERROR_MODULE_NETWORK + 16;
 const NS_ERROR_NET_RESET = NS_ERROR_MODULE_NETWORK + 20;
 const NS_ERROR_UNKNOWN_PROXY_HOST = NS_ERROR_MODULE_NETWORK + 42;
 const NS_ERROR_PROXY_CONNECTION_REFUSED = NS_ERROR_MODULE_NETWORK + 72;
@@ -110,7 +111,6 @@ BadCertHandler.prototype.QueryInterface =
 function badcert_queryinterface(aIID)
 {
     if (aIID.equals(Components.interfaces.nsIBadCertListener2) ||
-        aIID.equals(Components.interfaces.nsISSLErrorListener) ||
         aIID.equals(Components.interfaces.nsIInterfaceRequestor) ||
         aIID.equals(Components.interfaces.nsISupports))
     {
@@ -129,12 +129,6 @@ function badcert_notifyCertProblem(socketInfo, sslStatus, targetHost)
     return true;
 }
 
-BadCertHandler.prototype.notifySSLError =
-function badcert_notifySSLError(socketInfo, error, targetSite)
-{
-    return true;
-}
-
 /**
  * Wraps up various mechanics of sockets for easy consumption by other code.
  *
